@@ -43,10 +43,10 @@ type options = {
   inline : Declaremods.inline;
   uinstance : uinstanceoption;
   universe_decl : universe_decl_option;
-  nonuniform : bool option;
   reversible : bool option;
   keepunivs : bool option;
   redflags : CClosure.RedFlags.reds option;
+  no_tc: bool option;
 }
 
 type 'a coq_context = {
@@ -222,6 +222,13 @@ val isname : RawOpaqueData.t -> bool
 val nameout : RawOpaqueData.t -> Name.t
 val name : Name.t Conversion.t
 
+type global_or_pglobal =
+  | Global of term option
+  | PGlobal of term option * Univ.Instance.t option
+  | NotGlobal
+  | Var
+val is_global_or_pglobal : depth:int -> term -> global_or_pglobal
+
 val in_elpi_modpath : ty:bool -> Names.ModPath.t -> term
 val is_modpath : depth:int -> term -> bool
 val is_modtypath : depth:int -> term -> bool
@@ -277,6 +284,7 @@ val goals2query :
   in_elpi_tac_arg:(depth:int -> ?calldepth:int -> 'b coq_context -> hyp list -> Evd.evar_map -> State.t -> 'a -> State.t * term list * Conversion.extra_goals) ->
   depth:int -> State.t -> State.t * (Elpi.API.Ast.Loc.t * term) * Conversion.extra_goals
 val tclSOLUTION2EVD : Evd.evar_map -> 'a Elpi.API.Data.solution -> unit Proofview.tactic
+val solution2evd : Evd.evar_map -> 'a Elpi.API.Data.solution -> Evar.Set.t -> Evd.evar_map * Evar.t list * Evar.t list
 
 val show_coq_engine : ?with_univs:bool -> State.t -> string
 val show_coq_elpi_engine_mapping : State.t -> string
