@@ -96,8 +96,6 @@ let under_ctx name ty bo gterm2lp ~depth state x =
   let state = pop_env state in
   state, y, gl
 
-let type_gen = ref 0
-
 let is_hole x = match DAst.get x with GHole _ -> true | _ -> false
 
 let universe_level_name evd ({CAst.v=id} as lid) =
@@ -151,10 +149,8 @@ let rec gterm2lp ~depth state x =
   | GRef(gr, ul) when Global.is_polymorphic gr ->
     begin match ul with
     | None ->
-      incr type_gen;
-      let state, s =
-        API.RawQuery.mk_Arg state ~name:(Printf.sprintf "univ_inst_%d" !type_gen) ~args:[]
-      in
+      let state, f = F.Elpi.make state in
+      let s = API.RawData.mkUnifVar f ~args:[] state in
       state, in_elpi_poly_gr ~depth state gr s
     | Some l -> 
       let l' = List.map (glob_level x.CAst.loc state) l in
@@ -169,10 +165,10 @@ let rec gterm2lp ~depth state x =
             prlist_with_sep spc Id.print (Id.Map.bindings ctx.name2db |> List.map fst));
       state, E.mkConst (Id.Map.find id ctx.name2db)
   | GSort(UAnonymous {rigid=true}) ->
-      incr type_gen;
-      let state, s = API.RawQuery.mk_Arg state ~name:(Printf.sprintf "type_%d" !type_gen) ~args:[] in
+      let state, f = F.Elpi.make state in
+      let s = API.RawData.mkUnifVar f ~args:[] state in
       state, in_elpi_flex_sort s
-  | GSort(UNamed u) ->
+  | GSort(UNamed (None, u)) ->
       let env = get_glob_env state in
       in_elpi_sort ~depth state (sort_name env (get_sigma state) u)
   | GSort(_) -> nYI "(glob)HOAS for Type@{i j}"
@@ -198,7 +194,7 @@ let rec gterm2lp ~depth state x =
       let state, t, () = under_ctx name ty (Some bo) (nogls gterm2lp) ~depth state t in
       state, in_elpi_let name bo ty t
 
-  | GHole(_,_,Some arg) when !is_elpi_code arg ->
+  | GGenarg arg when !is_elpi_code arg ->
       let loc, text = !get_elpi_code arg in
       let s, x = Q.lp ~depth state loc text in
       let s, x =
@@ -212,7 +208,7 @@ let rec gterm2lp ~depth state x =
       in
 (*       Format.eprintf "unquote: %a\n" (Elpi_API.Extend.Pp.term depth) x; *)
         s, x
-  | GHole(_,_,Some arg) when !is_elpi_code_appArg arg ->
+  | GGenarg arg when !is_elpi_code_appArg arg ->
       begin match !get_elpi_code_appArg arg with
       | _, [] -> assert false
       | loc, hd :: vars ->
@@ -226,7 +222,9 @@ let rec gterm2lp ~depth state x =
             state, mkApp ~depth hd args
       end
 
-  | GHole (_,_,None) ->
+  | GGenarg _ -> nYI "(glob)HOAS for GGenarg"
+
+  | GHole _ ->
       let state, uv = F.Elpi.make state in
       let ctx, _ = Option.default (upcast @@ mk_coq_context ~options:(default_options ()) state, []) (get_ctx state) in
       let args =
@@ -238,8 +236,6 @@ let rec gterm2lp ~depth state x =
       in
       state, E.mkUnifVar uv ~args state
 
-  | GHole _ -> nYI "(glob)HOAS for GHole"
-
   | GCast(t,_,c_ty) ->
       let state, t = gterm2lp ~depth state t in
       let state, c_ty = gterm2lp ~depth state c_ty in
@@ -346,11 +342,13 @@ let rec gterm2lp ~depth state x =
       assert(List.length def <= 1);
       let bs = CList.init no_constructors (fun i ->
         let cno = i + 1 in
-        try CList.find_map (function
+        match CList.find_map (function
              | `Case((_,n as k),vars,bo) when n = cno -> Some (k,vars,bo)
              | `Case _ -> None
              | `Def _ -> assert false) bs
-        with Not_found ->
+        with
+        | Some v -> v
+        | None ->
           match def with
           | [`Def bo] ->
              let missing_k = ind,cno in
@@ -381,10 +379,12 @@ let rec gterm2lp ~depth state x =
   | GArray _ -> nYI "(glob)HOAS persistent arrays"
 ;;
 
+let lconstr_eoi = Pcoq.eoi_entry Pcoq.Constr.lconstr
+
 let coq_quotation ~depth state loc src =
   let ce =
     try
-      Pcoq.parse_string ~loc:(to_coq_loc loc) Pcoq.Constr.lconstr src
+      Pcoq.parse_string ~loc:(to_coq_loc loc) lconstr_eoi src
     with e ->
       CErrors.user_err
         Pp.(str(API.Ast.Loc.show loc) ++ spc() ++ CErrors.print_no_report e)
