@@ -43,10 +43,16 @@ let feedback_fmt_write, feedback_fmt_flush =
      Feedback.msg_notice Pp.(str s);
      Buffer.clear b)
 
+let elpi_cat = CWarnings.create_category ~name:"elpi" ()
+let elpi_depr_cat =
+  CWarnings.create_category
+    ~from:[elpi_cat;CWarnings.CoreCategories.deprecated]
+    ~name:"elpi.deprecated" ()
+
 let () = API.Setup.set_error (fun ?loc s -> err ?loc Pp.(str s))
 let () = API.Setup.set_anomaly (fun ?loc s -> err ?loc Pp.(str s))
 let () = API.Setup.set_type_error (fun ?loc s -> err ?loc Pp.(str s))
-let warn = CWarnings.create ~name:"runtime" ~category:"elpi" Pp.str
+let warn = CWarnings.create ~name:"runtime" ~category:elpi_cat Pp.str
 let () = API.Setup.set_warn (fun ?loc x -> warn ?loc:(Option.map to_coq_loc loc) x)
 let () = API.Setup.set_std_formatter (Format.make_formatter feedback_fmt_write feedback_fmt_flush)
 let () = API.Setup.set_err_formatter (Format.make_formatter feedback_fmt_write feedback_fmt_flush)
@@ -71,7 +77,7 @@ let safe_destApp sigma t =
 
 let mkGHole =
   DAst.make
-    (Glob_term.GHole(Evar_kinds.InternalHole,Namegen.IntroAnonymous,None))
+    (Glob_term.GHole(Evar_kinds.InternalHole,Namegen.IntroAnonymous))
     
 let mkApp ~depth t l =
   match l with
@@ -221,13 +227,13 @@ let detype ?(keepunivs=false) env sigma t =
   (* To avoid turning named universes into unnamed ones *)
   let options =
     if keepunivs then Flags.with_option Constrextern.print_universes
-    else (fun f x -> f x) in
+    else Flags.without_option Constrextern.print_universes in
   let gbody =
-    options (Detyping.detype Detyping.Now false Names.Id.Set.empty env sigma) t in
+    options (Detyping.detype Detyping.Now Names.Id.Set.empty env sigma) t in
   fix_detype gbody
 
 let detype_closed_glob env sigma closure =
-  let gbody = Detyping.detype_closed_glob false Names.Id.Set.empty env sigma closure in
+  let gbody = Detyping.detype_closed_glob Names.Id.Set.empty env sigma closure in
   fix_detype gbody
 
 type qualified_name = string list
@@ -269,3 +275,21 @@ let coq_version_parser version =
             !!major, !!minor, !!("-"^String.sub prerelease 5 (String.length prerelease - 5))
           else !!major, !!minor, -100
 
+
+let mp2path x =
+  let open Names in
+  let rec mp2sl = function
+    | MPfile dp -> CList.rev_map Id.to_string (DirPath.repr dp)
+    | MPbound id ->
+        let _,id,dp = MBId.repr id in
+        mp2sl (MPfile dp) @ [ Id.to_string id ]
+    | MPdot (mp,lbl) -> mp2sl mp @ [Label.to_string lbl] in
+  mp2sl x
+
+let gr2path gr =
+  let open Names in
+  match gr with
+  | Names.GlobRef.VarRef v -> mp2path (Safe_typing.current_modpath (Global.safe_env ()))
+  | Names.GlobRef.ConstRef c -> mp2path @@ Constant.modpath c
+  | Names.GlobRef.IndRef (i,_) -> mp2path @@ MutInd.modpath i
+  | Names.GlobRef.ConstructRef ((i,_),j) -> mp2path @@ MutInd.modpath i
