@@ -3,12 +3,18 @@
 (* ------------------------------------------------------------------------- *)
 
 open Coq_elpi_utils
+open Coq_elpi_programs
 
-type program_name = Loc.t * qualified_name
-
-val create_program : ?raw_args:bool -> program_name -> init:(Elpi.API.Ast.Loc.t * string) -> unit
+module Synterp : sig
+val create_program : ?raw_args:bool -> program_name -> unit
 val create_command : ?raw_args:bool -> program_name -> unit
 val create_tactic : program_name -> unit
+val create_db : program_name -> unit
+end
+
+val create_program : program_name -> init:(Elpi.API.Ast.Loc.t * string) -> unit
+val create_command : program_name -> unit
+val create_tactic : program_name -> unit
 val create_db : program_name -> init:(Elpi.API.Ast.Loc.t * string) -> unit
 
 val typecheck_program : ?program:qualified_name -> unit -> unit
@@ -45,3 +51,18 @@ val run_in_tactic : ?program:qualified_name -> Elpi.API.Ast.Loc.t * string -> Ge
 
 val export_command : qualified_name -> unit
 
+val atts2impl :
+  Elpi.API.Ast.Loc.t -> depth:int -> Elpi.API.State.t -> Attributes.vernac_flags ->
+    Elpi.API.Data.term -> Elpi.API.State.t * Elpi.API.Data.term
+val get_and_compile :
+  qualified_name -> Elpi.API.Compile.program * bool
+val run : static_check:bool ->
+    Elpi.API.Compile.program ->
+     [ `Ast of Elpi.API.Ast.query
+     | `Fun of
+         depth:int ->
+         Elpi.API.State.t ->
+         Elpi.API.State.t *
+         (Elpi.API.Ast.Loc.t * Elpi.API.Data.term) *
+         Elpi.API.Conversion.extra_goals ] ->
+    unit Elpi.API.Execute.outcome
\ No newline at end of file
